// Copyright 2019 Kina Wakasa
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'exception.dart';

/// A read operation, stands for a get method.
///
/// In general, you should use [ReadBatch] to construct it
/// instead of calling the constructor directly.
class ReadOp {
  final String key;

  ReadOp(this.key) {
    if (key == null) {
      throw BatchException('The key of read operation is null.');
    }
    if (key.isEmpty) {
      throw BatchException('The key of read operation is empty.');
    }
  }
}

/// A write operation, stands for a put or a delete method.
///
/// In general, you should use [WriteBatch] to construct it
/// instead of calling the constructor directly.
class WriteOp {
  final String key;

  final String value;

  WriteOp(this.key, this.value) {
    if (key == null) {
      throw BatchException('The key of write operation is null.');
    }
    if (key.isEmpty) {
      throw BatchException('The key of write operation is empty.');
    }
  }
}

/// A batch of ordered read operations.
class ReadBatch {
  /// The list of read operations.
  final ops = List<ReadOp>();

  /// Adds a get request to this [ReadBatch].
  void get(String key) => ops.add(ReadOp(key));
}

/// A batch of ordered write operations,
/// which either succeed together or fail together.
class WriteBatch {
  /// The list of write operations.
  final ops = List<WriteOp>();

  /// Adds a put request to this [WriteBatch].
  void put(String key, String value) => ops.add(WriteOp(key, value));

  /// Adds a delete request to this [WriteBatch]
  void delete(String key) => ops.add(WriteOp(key, null));
}
