// Copyright 2019 Kina Wakasa
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'dart:html';
import 'dart:indexed_db';

import 'batch.dart';
import 'persistent_lokv.dart';

/// A IndexedDB implementation of [PersistentLoKV]
class PersistentLoKVImpl extends PersistentLoKV {
  Database _database;

  PersistentLoKVImpl(String name) : super(name);

  @override
  Future<void> unsafeOpen() async {
    _database = await window.indexedDB.open(name, version: 1,
        onUpgradeNeeded: (VersionChangeEvent e) {
      e.target.result.createObjectStore('lokv');
    });
  }

  @override
  Future<void> unsafeClose() async {
    _database.close();
  }

  @override
  Future<void> unsafeDrop() async {
    await window.indexedDB.deleteDatabase(name);
  }

  @override
  Future<List<String>> unsafeRead(List<ReadOp> ops) async {
    var values = List<String>(ops.length);
    var transaction = _database.transaction(['lokv'], 'readonly');
    var objectStore = transaction.objectStore('lokv');
    for (var i = 0; i < ops.length; ++i) {
      values[i] = await objectStore.getObject(ops[i].key);
    }
    return values;
  }

  @override
  Future<void> unsafeWrite(List<WriteOp> ops) {
    var transaction = _database.transaction(['lokv'], 'readwrite');
    var objectStore = transaction.objectStore('lokv');
    for (var op in ops) {
      if (op.value == null) {
        objectStore.delete(op.key);
      } else {
        objectStore.put(op.value, op.key);
      }
    }
    return transaction.completed;
  }
}
