// Copyright 2019 Kina Wakasa
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:meta/meta.dart';

import 'batch.dart';
import 'exception.dart';

/// The status of a LoKV instance.
enum LoKVStatus { closed, open }

/// The abstract base class for a LoKV instance.
/// Subclass only need to implement all unsafe method,
/// and then they get various convenience methods for free.
abstract class LoKV {
  final String name;

  LoKVStatus _status;

  LoKVStatus get status => _status;

  LoKV(this.name) : _status = LoKVStatus.closed;

  /// Opens this LoKV instance without checking the status.
  @protected
  Future<void> unsafeOpen();

  /// Opens the LoKV instance.
  ///
  /// It will check the status of this Lokv instance,
  /// then call the [unsafeOpen] method.
  Future<void> open() async {
    if (name == null) {
      throw LoKVException('The name of LoKV instance is null.', this);
    }
    if (name.isEmpty) {
      throw LoKVException('The name of LoKV instance is empty.', this);
    }
    if (_status == LoKVStatus.closed) {
      await unsafeOpen();
      _status = LoKVStatus.open;
    }
  }

  /// Closes this LoKV instance without checking the status.
  @protected
  Future<void> unsafeClose();

  /// Closes the LoKV instance.
  ///
  /// It will check the status of this Lokv instance,
  /// then call the [unsafeClose] method.
  Future<void> close() async {
    if (_status == LoKVStatus.open) {
      await unsafeClose();
      _status = LoKVStatus.closed;
    }
  }

  /// Drops this LoKV instance without checking the status.
  @protected
  Future<void> unsafeDrop();

  /// Drops the LoKV instance.
  ///
  /// It will close this Lokv instance,
  /// then call the [unsafeDrop] method.
  Future<void> drop() async {
    await close();
    await unsafeDrop();
  }

  /// Performs the read operations in the list without checking the status,
  /// and return a list of values.
  @protected
  Future<List<String>> unsafeRead(List<ReadOp> ops);

  /// Performs the read operations in the batch.
  ///
  /// It will check whether the LoKV instance is open,
  /// then call the [unsafeRead] method.
  Future<List<String>> read(ReadBatch batch) {
    if (_status == LoKVStatus.closed) {
      throw LoKVException(
          'Attempt to perform a read operation in the closed state of the database.',
          this);
    }
    return unsafeRead(batch.ops);
  }

  /// Gets the value of the key.
  /// A null will return if the key does not exist.
  Future<String> get(String key) async =>
      (await read(ReadBatch()..get(key)))[0];

  /// Performs the write operations in the list without checking the status.
  @protected
  Future<void> unsafeWrite(List<WriteOp> ops);

  /// Performs the write operations in the batch.
  ///
  /// It will check whether the LoKV instance is open,
  /// then call the [unsafeWrite] method.
  Future<void> write(WriteBatch batch) {
    if (_status == LoKVStatus.closed) {
      throw LoKVException(
          'Attempt to perform a write operation in the closed state of the database.',
          this);
    }
    return unsafeWrite(batch.ops);
  }

  /// Puts the value into the key.
  /// The existing values will be replaced.
  /// If the value is null, the key will be deleted.
  Future<void> put(String key, String value) async =>
      write(WriteBatch()..put(key, value));

  /// Deletes the key.
  ///
  /// This method is equivalent to `put(key, null)`.
  Future<void> delete(String key) async => write(WriteBatch()..delete(key));
}
