// Copyright 2019 Kina Wakasa
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:lokv/lokv.dart';
import 'package:test/test.dart';

void main() {
  test('ReadOp', () {
    var op = ReadOp('key');
    expect(op.key, equals('key'));
    expect(() => ReadOp(null), throwsException);
    expect(() => ReadOp(''), throwsException);
  });
  test('WriteOp', () {
    var op = WriteOp('key', 'value');
    expect(op.key, equals('key'));
    expect(op.value, equals('value'));
    expect(() => WriteOp(null, 'value'), throwsException);
    expect(() => WriteOp('', 'value'), throwsException);
  });
  test('ReadBatch', () {
    var batch = ReadBatch();
    for (var i = 0; i < 10; ++i) {
      batch.get('key$i');
    }
    for (var i = 0; i < 10; ++i) {
      expect(batch.ops[i].key, equals('key$i'));
    }
    expect(() => batch.get(null), throwsException);
    expect(() => batch.get(''), throwsException);
  });
  test('WriteBatch', () {
    var batch = WriteBatch();
    for (var i = 0; i < 10; ++i) {
      if (i % 2 == 0) {
        batch.put('key$i', 'value$i');
      } else {
        batch.delete('key$i');
      }
    }
    for (var i = 0; i < 10; ++i) {
      expect(batch.ops[i].key, equals('key$i'));
      if (i % 2 == 0) {
        expect(batch.ops[i].value, equals('value$i'));
      } else {
        expect(batch.ops[i].value, equals(null));
      }
    }
    expect(() => batch.put(null, 'value'), throwsException);
    expect(() => batch.put('', 'value'), throwsException);
    expect(() => batch.delete(null), throwsException);
    expect(() => batch.delete(''), throwsException);
  });
  group('LoKV', () {
    var lokv = VolatileLoKV.create('lokv');
    setUp(() async {
      await lokv.open();
    });
    tearDown(() async {
      await lokv.drop();
    });
    test('Name', () {
      expect(lokv.name, equals('lokv'));
    });
    test('ReadWrite', () async {
      expect(await lokv.get('key1'), equals(null));
      expect(await lokv.get('key2'), equals(null));
      expect(await lokv.get('key3'), equals(null));
      await lokv.put('key1', 'value1');
      await lokv.put('key2', 'value2');
      await lokv.put('key3', 'value3');
      expect(await lokv.get('key1'), equals('value1'));
      expect(await lokv.get('key2'), equals('value2'));
      expect(await lokv.get('key3'), equals('value3'));
      await lokv.put('key1', 'value4');
      await lokv.delete('key2');
      await lokv.put('key3', 'value5');
      expect(await lokv.get('key1'), equals('value4'));
      expect(await lokv.get('key2'), equals(null));
      expect(await lokv.get('key3'), equals('value5'));
    });
    test('ReadWriteBatch', () async {
      expect(
          await lokv.read(ReadBatch()..get('key1')..get('key2')..get('key3')),
          equals([null, null, null]));
      await lokv.write(WriteBatch()
        ..put('key1', 'value1')
        ..put('key2', 'value2')
        ..put('key3', 'value3'));
      expect(
          await lokv.read(ReadBatch()..get('key1')..get('key2')..get('key3')),
          equals(['value1', 'value2', 'value3']));
      await lokv.write(WriteBatch()
        ..put('key1', 'value4')
        ..delete('key2')
        ..put('key3', 'value5'));
      expect(
          await lokv.read(ReadBatch()..get('key1')..get('key2')..get('key3')),
          equals(['value4', null, 'value5']));
    });
  });
  test('LoKVException', () {
    expect(VolatileLoKV.create(null).open, throwsException);
    expect(VolatileLoKV.create('').open, throwsException);
  });
}
